import {EventDispatcher} from "../base/EventDispatcher";
import { MOBILE_CLIENT } from "../settings";
import { FirstPerson } from "./firstPerson/firstPerson";
import { Orbit } from "./orbit/orbit";

const CAMERA_CONTROLLER_TYPE = {
    FIRST_PERSON: 0,
    ORBIT: 1,
}

const CameraController = function(scene, camera, domCanvasElement, domElement) {
    this.scene = scene;
    this.camera = camera;
    this.domElement = domElement;
    this.domCanvasElement = domCanvasElement;
    this.state = null;

    this.target = new THREE.Vector3();

    EventDispatcher.bind("CameraController.setActiveCameraController", this, this.setActiveCameraController);
    EventDispatcher.bind("CameraController.getActiveCameraController", this, function() {return this.state});

    this.cameraControllers = {};
    this.cameraControllers[CAMERA_CONTROLLER_TYPE.FIRST_PERSON] = new FirstPerson(this.scene, this.camera, this.domCanvasElement, this.domElement);
    this.cameraControllers[CAMERA_CONTROLLER_TYPE.ORBIT] = new Orbit(this.scene, this.camera, this.domCanvasElement);


    this.setActiveCameraController(CAMERA_CONTROLLER_TYPE.FIRST_PERSON, true);
    if (MOBILE_CLIENT) {
        this.setActiveCameraController(CAMERA_CONTROLLER_TYPE.FIRST_PERSON);
    }


    document.addEventListener('pointerlockchange', (e) => {
        EventDispatcher.send("CameraController.pointerlockchange", document.pointerLockElement !== this.domElement);
    });

    this.domCanvasElement.addEventListener("click", (e) => {
        if (this.state === CAMERA_CONTROLLER_TYPE.FIRST_PERSON) {
            EventDispatcher.send("FirstPerson.PointerLock.enable");
        }
    });

    document.addEventListener('pointerlockerror', (e) => {
        console.log(e);
    }, false);

}

CameraController.prototype = {

    constructor: CameraController,

    setActiveCameraController: function(type, start) {
        if (this.cameraControllers[this.state]) {
            if (this.state === CAMERA_CONTROLLER_TYPE.FIRST_PERSON) {
                const domElement = EventDispatcher.query("Engine.get.renderer").domElement;
                const point = this.scene.raycast({x: domElement.clientWidth / 2, y: domElement.clientHeight / 2});
                if (point.distance < 500) {
                    this.target = point.point;
                    this.target.y = 0;
                } else {
                    const dst = 30;
                    const target = new THREE.Vector3();
                    this.cameraControllers[this.state].camera.getWorldDirection(target);
                    target.multiplyScalar(dst);
                    target.add(this.cameraControllers[this.state].camera.position);

                    this.target = target;
                    this.target.y = 0;
                }
                
            } else {
                this.target = this.cameraControllers[this.state].controller.target.clone();
            }
            
            this.cameraControllers[this.state].remove();
        }
        this.state = type;
        if (this.cameraControllers[type]) {
            this.cameraControllers[type].init(this.target);
            EventDispatcher.send("CameraController.updateCameraControllerType", type, start);
        }
    },

    
}

export { CameraController, CAMERA_CONTROLLER_TYPE };