import {EventDispatcher} from "../../base/EventDispatcher";
import {FirstPersonControls} from "../../cameras/FirstPersonControls";
import { MOBILE_CLIENT } from "../../settings";

const FirstPerson = function (scene, camera, domCanvasElement, domElement) {
    this.scene = scene;
    this.camera = camera;
    this.domElement = domElement;
    this.domCanvasElement = domCanvasElement;
    this.controller = null;
}

FirstPerson.prototype = {

    constructor: FirstPerson,

    init(target) {
        this.controller = new FirstPersonControls(this.camera, this.domCanvasElement, this.domElement);
        this.controller.movementSpeed = 25;
        this.controller.lookSpeed = 0.05;
        this.controller.initEvent();
        this.controller.target = target;
        this.camera.lookAt(target.x, target.y, target.z);
        this.scene.add(this.controller);

        EventDispatcher.bind("Input.enable", this, this.inputEnable);
        EventDispatcher.bind("Input.disable", this, this.inputDisable);

        EventDispatcher.bind("FirstPerson.PointerLock.enable", this, this.pointerLockEnable);
        EventDispatcher.bind("FirstPerson.PointerLock.disable", this, this.pointerLockDisable);
    },
    remove() {
        this.controller.removeEvent();
        this.scene.remove(this.controller);

        EventDispatcher.unbind("Input.enable", this.inputEnable);
        EventDispatcher.unbind("Input.disable", this.inputDisable);

        EventDispatcher.unbind("FirstPerson.PointerLock.enable", this.pointerLockEnable);
        EventDispatcher.unbind("FirstPerson.PointerLock.disable", this.pointerLockDisable);
    },

    inputEnable() {
        this.controller.enabled = true
    },

    inputDisable() {
        this.controller.enabled = false
    },

    pointerLockEnable() {
        if (MOBILE_CLIENT) {
            return;
        }
        this.domElement.requestPointerLock();
    },

    pointerLockDisable() {
        if (MOBILE_CLIENT) {
            return;
        }
        document.exitPointerLock();
    },

}

export {FirstPerson};