import {EventDispatcher} from "../../base/EventDispatcher";
import {CameraControllerOrbit} from "../../cameras/CameraControllerOrbit";

const Orbit = function(scene, camera, domElement) {
    this.scene = scene;
    this.camera = camera;
    this.domElement = domElement;
    this.controller = null;


}

Orbit.prototype = {

    constructor: Orbit,

    init(target) {
        this.controller = new CameraControllerOrbit(this.camera, this.domElement);
        this.controller.maxPolarAngle = Math.PI;
        this.controller.minDistance = 1;
        this.controller.maxDistance = 100;
        this.controller.target = target;
        this.camera.lookAt(target.x, target.y, target.z);
        this.controller.initEvent();
        this.scene.add(this.controller);

        EventDispatcher.bind("Input.enable", this, this.inputEnable);
        EventDispatcher.bind("Input.disable", this, this.inputDisable);
    },
    remove() {
        EventDispatcher.unbind("Input.enable", this.inputEnable);
        EventDispatcher.unbind("Input.disable", this.inputDisable);

        this.controller.removeEvent();
        this.scene.remove(this.controller);
    },

    inputEnable() {
        this.controller.enabled = true
    },

    inputDisable() {
        this.controller.enabled = false
    },

}

export { Orbit };