/**
 * @author qiao / https://github.com/qiao
 * @author mrdoob / http://mrdoob.com
 * @author alteredq / http://alteredqualia.com/
 * @author WestLangley / http://github.com/WestLangley
 * @author erich666 / http://erichaines.com
 * @author vgaidukov / https://github.com/vladgaidukov
 */
import {EventDispatcher} from "../base/EventDispatcher.js";
import {
    MOBILE_CLIENT
} from "../settings.js";

function CameraControllerOrbit(object, domElement) {

    this.object = object;

    this.domElement = (domElement !== undefined) ? domElement : document;

    // Set to false to disable this control
    this.enabled = true;

    // "target" sets the location of focus, where the object orbits around
    this.target = new THREE.Vector3();

    // How far you can dolly in and out ( PerspectiveCamera only )
    this.minDistance = 0;
    this.maxDistance = Infinity;

    // How far you can zoom in and out ( OrthographicCamera only )
    this.minZoom = 0;
    this.maxZoom = Infinity;

    // How far you can orbit vertically, upper and lower limits.
    // Range is 0 to Math.PI radians.
    this.minPolarAngle = 0; // radians
    this.maxPolarAngle = Math.PI; // radians

    // How far you can orbit horizontally, upper and lower limits.
    // If set, must be a sub-interval of the interval [ - Math.PI, Math.PI ].
    this.minAzimuthAngle = -Infinity; // radians
    this.maxAzimuthAngle = Infinity; // radians

    // Set to true to enable damping (inertia)
    // If damping is enabled, you must call controls.update() in your animation loop
    this.enableDamping = true;
    // this.dampingFactor = 0.25;
    this.dampingFactor = 0.5;

    // This option actually enables dollying in and out; left as "zoom" for backwards compatibility.
    // Set to false to disable zooming
    this.enableZoom = true;
    this.zoomSpeed = 1.0;

    // Set to false to disable rotating
    this.enableRotate = true;
    this.rotateSpeed = 1.0;

    // Set to false to disable panning
    this.enablePan = true;
    this.keyPanSpeed = 60.0; // pixels moved per arrow key push, default 7
    this.keySpeed = 1600.0;
    this.targetSpeed = 12.5;
    this.screenSpacePanning = true;
    this.horizontalRotatingSpeed = 140;
    this.verticalRotatingSpeed = 180;
    this.targetDistance = 22.8;

    // Set to true to automatically rotate around the target
    // If auto-rotate is enabled, you must call controls.update() in your animation loop
    this.autoRotate = false;
    this.autoRotateSpeed = 2.0; // 30 seconds per round when fps is 60

    // Set to false to disable use of the keys
    this.enableKeys = true;

    // The four arrow keys
    this.keys = {
        LEFT: "KeyA", 
        UP: "KeyW", 
        RIGHT: "KeyD", 
        BOTTOM: "KeyS", 
        TARGET_UP: "KeyE", 
        TARGET_DOWN: "KeyQ",
        ARROW_LEFT: 'ArrowLeft', 
        ARROW_UP: 'ArrowUp', 
        ARROW_RIGHT: 'ArrowRight', 
        ARROW_BOTTOM: 'ArrowDown',
    };

    this.enableControllerKey = {ROTATE: "ShiftLeft"};

    // Mouse buttons
    this.mouseButtons = {ORBIT: THREE.MOUSE.LEFT, ZOOM: THREE.MOUSE.MIDDLE, PAN: THREE.MOUSE.RIGHT};

    // for reset
    this.target0 = this.target.clone();
    this.position0 = this.object.position.clone();
    this.zoom0 = this.object.zoom;

    this.getPolarAngle = function() {

        return spherical.phi;

    };

    this.getAzimuthalAngle = function() {

        return spherical.theta;

    };

    this.saveState = function() {

        scope.target0.copy(scope.target);
        scope.position0.copy(scope.object.position);
        scope.zoom0 = scope.object.zoom;

    };

    this.reset = function() {

        scope.target.copy(scope.target0);
        scope.object.position.copy(scope.position0);
        scope.object.zoom = scope.zoom0;

        scope.object.updateProjectionMatrix();


        state = STATE.NONE;

    };

    this.focusOut = function() {
        this.moveForward = false;
        this.moveLeft = false;
        this.moveBackward = false;
        this.moveRight = false;
        this.moveUp = false;
        this.moveDown = false;
        this.rotatingLeft = false;
        this.rotatingRight = false;
        this.rotatingUp = false;
        this.rotatingBottom = false;
    }

    this.update = function(delta) {
        if (scope.isMovement) {
            this._update(delta);
        }
    }
    this._update = function() {
        
        var offset = new THREE.Vector3();

        var quat = new THREE.Quaternion().setFromUnitVectors(object.up, new THREE.Vector3(0, 1, 0));
        var quatInverse = quat.clone().invert();

        var lastPosition = new THREE.Vector3();
        var lastQuaternion = new THREE.Quaternion();

        return function update(delta) {

            if (scope.enabled === false) return;

            const coord = EventDispatcher.query("MouseEventsHandler.cursorPosition");

            if (this.moveForward) {
                scope.screenSpacePanning = false;
                pan(0, scope.keySpeed * delta);
            }

            if (this.moveBackward) {
                scope.screenSpacePanning = false;
                pan(0, -scope.keySpeed * delta);
            }

            if (this.moveLeft) {
                pan(scope.keySpeed * delta, 0);
            }

            if (this.moveRight) {
                pan(-scope.keySpeed * delta, 0);
            }

            if (this.moveUp) {
                // scope.screenSpacePanning = true;
                // pan(0, scope.keySpeed * delta);
                scope.target.y += scope.targetSpeed * delta;
                scope.object.position.y += scope.targetSpeed * delta;
            }

            if (this.moveDown) {
                // scope.screenSpacePanning = true;
                // pan(0, -scope.keySpeed * delta);
                scope.target.y -= scope.targetSpeed * delta;
                scope.object.position.y -= scope.targetSpeed * delta;
            }

            if (this.rotatingUp) {
                handleMouseDownRotate({x: 0, y: 0});
                handleMouseMoveRotate({x: 0, y: this.horizontalRotatingSpeed * delta}, false);
            }

            if (this.rotatingBottom) {
                handleMouseDownRotate({x: 0, y: 0});
                handleMouseMoveRotate({x: 0, y: -this.horizontalRotatingSpeed * delta}, false);
            }

            if (this.rotatingLeft) {
                handleMouseDownRotate({x: 0, y: 0});
                handleMouseMoveRotate({x: -this.verticalRotatingSpeed * delta, y: 0}, false);
            }

            if (this.rotatingRight) {
                handleMouseDownRotate({x: 0, y: 0});
                handleMouseMoveRotate({x: this.verticalRotatingSpeed * delta, y: 0}, false);
            }

            EventDispatcher.send("Camera.update", this.object, this.target, coord);

            var position = scope.object.position;

            offset.copy(position).sub(scope.target);

            offset.applyQuaternion(quat);

            spherical.setFromVector3(offset);

            if (scope.autoRotate && state === STATE.NONE) {

                rotateLeft(getAutoRotationAngle());

            }

            spherical.theta += sphericalDelta.theta;
            spherical.phi += sphericalDelta.phi;

            spherical.theta = Math.max(scope.minAzimuthAngle, Math.min(scope.maxAzimuthAngle, spherical.theta));

            spherical.phi = Math.max(scope.minPolarAngle, Math.min(scope.maxPolarAngle, spherical.phi));

            spherical.makeSafe();

            spherical.radius *= scale;

            spherical.radius = Math.max(scope.minDistance, Math.min(scope.maxDistance, spherical.radius));

            scope.target.add(panOffset);

            offset.setFromSpherical(spherical);

            offset.applyQuaternion(quatInverse);

            position.copy(scope.target).add(offset);

            scope.object.lookAt(scope.target);

            if (scope.enableDamping === true) {

                sphericalDelta.theta *= (1 - scope.dampingFactor);
                sphericalDelta.phi *= (1 - scope.dampingFactor);

            }
            else {

                sphericalDelta.set(0, 0, 0);

            }

            scale = 1;
            panOffset.set(0, 0, 0);

            if (zoomChanged ||
                lastPosition.distanceToSquared(scope.object.position) > EPS ||
                8 * (1 - lastQuaternion.dot(scope.object.quaternion)) > EPS) {

                lastPosition.copy(scope.object.position);
                lastQuaternion.copy(scope.object.quaternion);
                zoomChanged = false;

                return true;

            }

            return false;

        };

    }();

    this.deactivate = function() {

        if (MOBILE_CLIENT) {
            EventDispatcher.unbind("mouse.down", onTouchStart);
            EventDispatcher.unbind("mouse.up", onTouchEnd);
            EventDispatcher.unbind("mouse.move", onTouchMove);
        }
        else {

            EventDispatcher.unbind("mouse.down", onMouseDown);
            EventDispatcher.unbind("mouse.scroll", onMouseWheel);
        }

    };

    this.isMoving = function () {
        if (
            scope.moveForward ||
            scope.moveLeft ||
            scope.moveBackward ||
            scope.moveRight ||
            scope.moveUp ||
            scope.moveDown ||
            scope.rotatingLeft ||
            scope.rotatingRight ||
            scope.rotatingUp ||
            scope.rotatingBottom
            ) {
            scope.isMovement = true;
        } else {
            scope.isMovement = false;
        }
    }

    this.onKeyDownMoveForward = function () {
        // scope.screenSpacePanning = false;
		scope.moveForward = true;
        this.isMoving();
	}

	this.onKeyUpMoveForward = function () {
		scope.moveForward = false;
        this.isMoving();
	}

	this.onKeyDownMoveLeft = function () {
		scope.moveLeft = true;
        this.isMoving();
	}

	this.onKeyUpMoveLeft = function () {
		scope.moveLeft = false;
        this.isMoving();
	}

	this.onKeyDownMoveBackward = function () {
        // scope.screenSpacePanning = false;
		scope.moveBackward = true;
        this.isMoving();
	}

	this.onKeyUpMoveBackward = function () {
		scope.moveBackward = false;
        this.isMoving();
	}

	this.onKeyDownMoveRight = function () {
		scope.moveRight = true;
        this.isMoving();
	}

	this.onKeyUpMoveRight = function () {
		scope.moveRight = false;
        this.isMoving();
	}

	this.onKeyDownMoveUp = function (e) {
        // scope.screenSpacePanning = true;
		scope.moveUp = true;
        this.isMoving();
	}

	this.onKeyUpMoveUp = function (e) {
        // scope.screenSpacePanning = false;
		scope.moveUp = false;
        this.isMoving();
	}

	this.onKeyDownMoveDown = function (e) {
        // scope.screenSpacePanning = true;
		scope.moveDown = true;
        this.isMoving();
	}

	this.onKeyUpMoveDown = function (e) {
        // scope.screenSpacePanning = false;
		scope.moveDown = false;
        this.isMoving();
	}

    this.onKeyDownRotateUp = function (e) {
        scope.rotatingUp = true;
        this.isMoving();
    }

    this.onKeyUpRotateUp = function (e) {
        scope.rotatingUp = false;
        this.isMoving();
    }

    this.onKeyDownRotateBottom = function (e) {
        scope.rotatingBottom = true;
        this.isMoving();
    }

    this.onKeyUpRotateBottom = function (e) {
        scope.rotatingBottom = false;
        this.isMoving();
    }

    this.onKeyDownRotateLeft = function (e) {
        scope.rotatingLeft = true;
        this.isMoving();
    }

    this.onKeyUpRotateLeft = function (e) {
        scope.rotatingLeft = false;
        this.isMoving();
    }

    this.onKeyDownRotateRight = function (e) {
        scope.rotatingRight = true;
        this.isMoving();
    }

    this.onKeyUpRotateRight= function (e) {
        scope.rotatingRight = false;
        this.isMoving();
    }

    this.setTarget = function (vec) {
        scope.isMovement = true;
        scope.maxDistance = 20;
        new TWEEN.Tween(scope.target)
            .to(vec, 250)
            .easing(TWEEN.Easing.Cubic.Out)
            //.onUpdate(() => render())
            .start()
            .onComplete(() => {
                scope.isMovement = false;
                scope.maxDistance = 100;
            });
    }

    var scope = this;

    var changeEvent = {type: "change"};
    var startEvent = {type: "start"};
    var endEvent = {type: "end"};

    var STATE = {NONE: -1, ROTATE: 0, DOLLY: 1, PAN: 2, TOUCH_ROTATE: 3, TOUCH_DOLLY: 4, TOUCH_PAN: 5};

    var state = STATE.NONE;

    var EPS = 0.000001;

    var spherical = new THREE.Spherical();
    var sphericalDelta = new THREE.Spherical();

    var scale = 1;
    var panOffset = new THREE.Vector3();
    var zoomChanged = false;

    var rotateStart = new THREE.Vector2();
    var rotateEnd = new THREE.Vector2();
    var rotateDelta = new THREE.Vector2();

    var panStart = new THREE.Vector2();
    var panEnd = new THREE.Vector2();
    var panDelta = new THREE.Vector2();

    var dollyStart = new THREE.Vector2();
    var dollyEnd = new THREE.Vector2();
    var dollyDelta = new THREE.Vector2();

    function getAutoRotationAngle() {

        return 2 * Math.PI / 60 / 60 * scope.autoRotateSpeed;

    }

    function getZoomScale() {

        return Math.pow(0.95, scope.zoomSpeed);

    }

    function rotateLeft(angle) {

        sphericalDelta.theta -= angle;

    }

    function rotateUp(angle) {

        sphericalDelta.phi -= angle;

    }

    var panLeft = function() {

        var v = new THREE.Vector3();

        return function panLeft(distance, objectMatrix) {

            v.setFromMatrixColumn(objectMatrix, 0);
            v.multiplyScalar(-distance);

            panOffset.add(v);

        };

    }();

    var panUp = function() {

        var v = new THREE.Vector3();

        return function panUp(distance, objectMatrix) {

            // // v.setFromMatrixColumn(objectMatrix, 1);
            // // v.multiplyScalar(distance);

            // v.setFromMatrixColumn( objectMatrix, 0 );
			// v.crossVectors( scope.object.up, v );
            // v.multiplyScalar(distance);

            // panOffset.add(v);
            if ( scope.screenSpacePanning === true ) {

                v.setFromMatrixColumn( objectMatrix, 1 );

            } else {

                v.setFromMatrixColumn( objectMatrix, 0 );
                v.crossVectors( scope.object.up, v );

            }

            v.multiplyScalar( distance );

            panOffset.add( v );

        };

    }();

    var pan = function() {

        var offset = new THREE.Vector3();

        return function pan(deltaX, deltaY) {

            var element = scope.domElement === document ? scope.domElement.body : scope.domElement;

            if (scope.object instanceof THREE.PerspectiveCamera) {

                // var position = scope.object.position;
                // offset.copy(position).sub(scope.target);
                // var targetDistance = offset.length();
                var targetDistance = scope.targetDistance;
                targetDistance *= Math.tan((scope.object.fov / 2) * Math.PI / 180.0);

                panLeft(2 * deltaX * targetDistance / element.clientHeight, scope.object.matrix);
                panUp(2 * deltaY * targetDistance / element.clientHeight, scope.object.matrix);

            }
            else if (scope.object instanceof THREE.OrthographicCamera) {

                panLeft(deltaX * (scope.object.right - scope.object.left) / scope.object.zoom / element.clientWidth, scope.object.matrix);
                panUp(deltaY * (scope.object.top - scope.object.bottom) / scope.object.zoom / element.clientHeight, scope.object.matrix);

            }
            else {

                console.warn("WARNING: CameraControllerOrbit.js encountered an unknown camera type - pan disabled.");
                scope.enablePan = false;

            }

        };

    }();

    function dollyIn(dollyScale) {

        if (scope.object instanceof THREE.PerspectiveCamera) {

            scale /= dollyScale;

        }
        else if (scope.object instanceof THREE.OrthographicCamera) {

            scope.object.zoom = Math.max(scope.minZoom, Math.min(scope.maxZoom, scope.object.zoom * dollyScale));
            scope.object.updateProjectionMatrix();
            zoomChanged = true;

        }
        else {

            console.warn("WARNING: CameraControllerOrbit.js encountered an unknown camera type - dolly/zoom disabled.");
            scope.enableZoom = false;

        }

    }

    function dollyOut(dollyScale) {

        if (scope.object instanceof THREE.PerspectiveCamera) {

            scale *= dollyScale;

        }
        else if (scope.object instanceof THREE.OrthographicCamera) {

            scope.object.zoom = Math.max(scope.minZoom, Math.min(scope.maxZoom, scope.object.zoom / dollyScale));
            scope.object.updateProjectionMatrix();
            zoomChanged = true;

        }
        else {

            console.warn("WARNING: CameraControllerOrbit.js encountered an unknown camera type - dolly/zoom disabled.");
            scope.enableZoom = false;

        }

    }

    function handleMouseDownRotate(event) {

        rotateStart.set(event.x, event.y);

    }

    function handleMouseDownDolly(event) {

        dollyStart.set(event.x, event.y);

    }

    function handleMouseDownPan(event) {

        panStart.set(event.x, event.y);

    }

    function handleMouseMoveRotate(event, update=true) {

        rotateEnd.set(event.x, event.y);
        rotateDelta.subVectors(rotateEnd, rotateStart);

        var element = scope.domElement === document ? scope.domElement.body : scope.domElement;

        rotateLeft(2 * Math.PI * rotateDelta.x / element.clientWidth * scope.rotateSpeed);

        rotateUp(2 * Math.PI * rotateDelta.y / element.clientHeight * scope.rotateSpeed);

        rotateStart.copy(rotateEnd);

        if (update) {
            scope._update(0.016);
        }
    }

    function handleMouseMoveDolly(event) {

        dollyEnd.set(event.x, event.y);

        dollyDelta.subVectors(dollyEnd, dollyStart);

        if (dollyDelta.y > 0) {

            dollyIn(getZoomScale());

        }
        else if (dollyDelta.y < 0) {

            dollyOut(getZoomScale());

        }

        dollyStart.copy(dollyEnd);


    }

    function handleMouseMovePan(event) {
        scope.screenSpacePanning = true;
        panEnd.set(event.x, event.y);

        panDelta.subVectors(panEnd, panStart);

        pan(panDelta.x, panDelta.y);

        panStart.copy(panEnd);

        scope._update(0.016);
    }

    function handleMouseWheel(dir) {

        if (dir < 0) {

            dollyOut(getZoomScale());

        }
        else if (dir > 0) {

            dollyIn(getZoomScale());

        }

        
        scope._update(0.016);
    }

    function handleTouchStartRotate(event) {

        rotateStart.set(event.touches[ 0 ].pageX, event.touches[ 0 ].pageY);

    }

    function handleTouchStartDolly(event) {

        var dx = event.touches[ 0 ].pageX - event.touches[ 1 ].pageX;
        var dy = event.touches[ 0 ].pageY - event.touches[ 1 ].pageY;

        var distance = Math.sqrt(dx * dx + dy * dy);

        dollyStart.set(0, distance);

    }

    function handleTouchStartPan(event) {

        panStart.set(event.touches[ 0 ].pageX, event.touches[ 0 ].pageY);

    }

    function handleTouchMoveRotate(event) {

        rotateEnd.set(event.touches[ 0 ].pageX, event.touches[ 0 ].pageY);
        rotateDelta.subVectors(rotateEnd, rotateStart);

        var element = scope.domElement === document ? scope.domElement.body : scope.domElement;
        rotateLeft(2 * Math.PI * rotateDelta.x / element.clientWidth * scope.rotateSpeed);

        rotateUp(2 * Math.PI * rotateDelta.y / element.clientHeight * scope.rotateSpeed);

        rotateStart.copy(rotateEnd);

        

    }

    function handleTouchMoveDolly(event) {

        var dx = event.touches[ 0 ].pageX - event.touches[ 1 ].pageX;
        var dy = event.touches[ 0 ].pageY - event.touches[ 1 ].pageY;

        var distance = Math.sqrt(dx * dx + dy * dy);
        dollyEnd.set(0, distance);

        dollyDelta.subVectors(dollyEnd, dollyStart);

        if (dollyDelta.y > 0) {

            dollyOut(getZoomScale());

        }
        else if (dollyDelta.y < 0) {

            dollyIn(getZoomScale());

        }

        dollyStart.copy(dollyEnd);

        

    }

    function handleTouchMovePan(event) {

        panEnd.set(event.touches[ 0 ].pageX, event.touches[ 0 ].pageY);

        panDelta.subVectors(panEnd, panStart);

        pan(panDelta.x, panDelta.y);

        panStart.copy(panEnd);

        

    }

    function onMouseDown(event) {

        if (scope.enabled === false) return;

        switch (event.button) {

        case scope.mouseButtons.ORBIT:

            if (scope.enableRotate === false) return;

            handleMouseDownRotate(event);

            state = STATE.ROTATE;

            break;

        case scope.mouseButtons.ZOOM:

            if (scope.enableZoom === false) return;

            handleMouseDownDolly(event);

            state = STATE.DOLLY;

            break;

        case scope.mouseButtons.PAN:

            if (scope.enablePan === false) return;

            handleMouseDownPan(event);

            state = STATE.PAN;

            break;

        }

        if (state !== STATE.NONE) {

            EventDispatcher.bind("mouse.move", this, onMouseMove);
            EventDispatcher.bind("mouse.up", this, onMouseUp);

        }

    }

    function onMouseMove(event) {

        scope.rotatingLeft = false;
        scope.rotatingRight = false;
        scope.rotatingUp = false;
        scope.rotatingBottom = false;

        if (scope.enabled === false) return;

        switch (state) {

        case STATE.ROTATE:

            if (scope.enableRotate === false) return;

            handleMouseMoveRotate(event);

            break;

        case STATE.DOLLY:

            if (scope.enableZoom === false) return;

            handleMouseMoveDolly(event);

            break;

        case STATE.PAN:

            if (scope.enablePan === false) return;

            handleMouseMovePan(event);

            break;

        }

    }

    function onMouseUp(event) {

        if (scope.enabled === false) return;

        EventDispatcher.unbind("mouse.move", onMouseMove);
        EventDispatcher.unbind("mouse.up", onMouseUp);

        state = STATE.NONE;

    }

    function onMouseWheel(dir) {

        if (scope.enabled === false || scope.enableZoom === false || (state !== STATE.NONE && state !== STATE.ROTATE)) return;

        handleMouseWheel(dir);
    }

    function onKeyDown(event) {

        if (scope.enabled === false || scope.enableKeys === false || scope.enablePan === false) return;
        handleKeyDown(event);

    }

    function onTouchStart(event) {

        if (scope.enabled === false) return;

        event = event.event;

        switch (event.touches.length) {

        case 1:

            if (scope.enableRotate === false) return;

            handleTouchStartRotate(event);

            state = STATE.TOUCH_ROTATE;

            break;

        case 2:

            if (scope.enableZoom === false) return;

            handleTouchStartDolly(event);

            state = STATE.TOUCH_DOLLY;

            break;

        case 3:

            if (scope.enablePan === false) return;

            handleTouchStartPan(event);

            state = STATE.TOUCH_PAN;

            break;

        default:

            state = STATE.NONE;

        }

    }

    function onTouchMove(event) {

        if (scope.enabled === false) return;

        event = event.event;

        switch (event.touches.length) {

        case 1:

            if (scope.enableRotate === false) return;
            if (state !== STATE.TOUCH_ROTATE) return;

            handleTouchMoveRotate(event);

            break;

        case 2:

            if (scope.enableZoom === false) return;
            if (state !== STATE.TOUCH_DOLLY) return;

            handleTouchMoveDolly(event);

            break;

        case 3:

            if (scope.enablePan === false) return;
            if (state !== STATE.TOUCH_PAN) return;

            handleTouchMovePan(event);

            break;

        default:

            state = STATE.NONE;

        }

    }

    function onTouchEnd(event) {

        if (scope.enabled === false) return;

        state = STATE.NONE;

    }

    this.onKeyDownEnableRotate = function () {
		scope.enableRotate = false;
	}
    this.onKeyUpEnableRotate = function () {
		scope.enableRotate = true;
	}

    this.removeEvent = function() {

        if (MOBILE_CLIENT) {
            EventDispatcher.unbind("mouse.down", onTouchStart);
            EventDispatcher.unbind("mouse.up", onTouchEnd);
            EventDispatcher.unbind("mouse.move", onTouchMove);
        }
        else {

            EventDispatcher.unbind("mouse.down", onMouseDown);
            EventDispatcher.unbind("mouse.scroll", onMouseWheel);

            EventDispatcher.unbind("keyboard.keydown." + scope.enableControllerKey.ROTATE, this.onKeyDownEnableRotate);
            EventDispatcher.unbind("keyboard.keyup." + scope.enableControllerKey.ROTATE, this.onKeyUpEnableRotate);

            EventDispatcher.unbind("keyboard.keydown." + scope.keys.UP, this.onKeyDownMoveForward);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.UP, this.onKeyUpMoveForward);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.BOTTOM, this.onKeyDownMoveBackward);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.BOTTOM, this.onKeyUpMoveBackward);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.LEFT, this.onKeyDownMoveLeft);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.LEFT, this.onKeyUpMoveLeft);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.RIGHT, this.onKeyDownMoveRight);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.RIGHT, this.onKeyUpMoveRight);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.TARGET_UP, this.onKeyDownMoveUp);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.TARGET_UP, this.onKeyUpMoveUp);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.TARGET_DOWN, this.onKeyDownMoveDown);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.TARGET_DOWN, this.onKeyUpMoveDown);

            EventDispatcher.unbind("keyboard.keydown." + scope.keys.ARROW_UP, this.onKeyDownRotateUp);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.ARROW_UP, this.onKeyUpRotateUp);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.ARROW_BOTTOM, this.onKeyDownRotateBottom);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.ARROW_BOTTOM, this.onKeyUpRotateBottom);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.ARROW_LEFT, this.onKeyDownRotateLeft);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.ARROW_LEFT, this.onKeyUpRotateLeft);
            EventDispatcher.unbind("keyboard.keydown." + scope.keys.ARROW_RIGHT, this.onKeyDownRotateRight);
            EventDispatcher.unbind("keyboard.keyup." + scope.keys.ARROW_RIGHT, this.onKeyUpRotateRight);

            EventDispatcher.unbind("CameraControllerOrbit.setTarget", this.setTarget);
            EventDispatcher.unbind("CameraControllerOrbit.focusOut", this.focusOut);
        }

    };

    this.initEvent = function() {
        if (MOBILE_CLIENT) {
            EventDispatcher.bind("mouse.down", this, onTouchStart);
            EventDispatcher.bind("mouse.up", this, onTouchEnd);
            EventDispatcher.bind("mouse.move", this, onTouchMove);
        }
        else {
            EventDispatcher.bind("mouse.down", this, onMouseDown);
            EventDispatcher.bind("mouse.scroll", this, onMouseWheel);
            
            EventDispatcher.bind("keyboard.keydown." + scope.enableControllerKey.ROTATE, this, this.onKeyDownEnableRotate);
            EventDispatcher.bind("keyboard.keyup." + scope.enableControllerKey.ROTATE, this, this.onKeyUpEnableRotate);

            EventDispatcher.bind("keyboard.keydown." + scope.keys.UP, this, this.onKeyDownMoveForward);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.UP, this, this.onKeyUpMoveForward);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.BOTTOM, this, this.onKeyDownMoveBackward);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.BOTTOM, this, this.onKeyUpMoveBackward);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.LEFT, this, this.onKeyDownMoveLeft);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.LEFT, this, this.onKeyUpMoveLeft);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.RIGHT, this, this.onKeyDownMoveRight);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.RIGHT, this, this.onKeyUpMoveRight);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.TARGET_UP, this, this.onKeyDownMoveUp);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.TARGET_UP, this, this.onKeyUpMoveUp);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.TARGET_DOWN, this, this.onKeyDownMoveDown);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.TARGET_DOWN, this, this.onKeyUpMoveDown);

            EventDispatcher.bind("keyboard.keydown." + scope.keys.ARROW_UP, this, this.onKeyDownRotateUp);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.ARROW_UP, this, this.onKeyUpRotateUp);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.ARROW_BOTTOM, this, this.onKeyDownRotateBottom);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.ARROW_BOTTOM, this, this.onKeyUpRotateBottom);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.ARROW_LEFT, this, this.onKeyDownRotateLeft);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.ARROW_LEFT, this, this.onKeyUpRotateLeft);
            EventDispatcher.bind("keyboard.keydown." + scope.keys.ARROW_RIGHT, this, this.onKeyDownRotateRight);
            EventDispatcher.bind("keyboard.keyup." + scope.keys.ARROW_RIGHT, this, this.onKeyUpRotateRight);

            EventDispatcher.bind("CameraControllerOrbit.setTarget", this, this.setTarget);
            EventDispatcher.bind("CameraControllerOrbit.focusOut", this, this.focusOut);
        }
    }


}

CameraControllerOrbit.prototype = {
    center: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .center has been renamed to .target");
            return this.target;

        }

    },

    noZoom: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .noZoom has been deprecated. Use .enableZoom instead.");
            return !this.enableZoom;

        },

        set: function(value) {

            console.warn("VG.CameraControllerOrbit: .noZoom has been deprecated. Use .enableZoom instead.");
            this.enableZoom = !value;

        }

    },

    noRotate: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .noRotate has been deprecated. Use .enableRotate instead.");
            return !this.enableRotate;

        },

        set: function(value) {

            console.warn("VG.CameraControllerOrbit: .noRotate has been deprecated. Use .enableRotate instead.");
            this.enableRotate = !value;

        }

    },

    noPan: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .noPan has been deprecated. Use .enablePan instead.");
            return !this.enablePan;

        },

        set: function(value) {

            console.warn("VG.CameraControllerOrbit: .noPan has been deprecated. Use .enablePan instead.");
            this.enablePan = !value;

        }

    },

    noKeys: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .noKeys has been deprecated. Use .enableKeys instead.");
            return !this.enableKeys;

        },

        set: function(value) {

            console.warn("VG.CameraControllerOrbit: .noKeys has been deprecated. Use .enableKeys instead.");
            this.enableKeys = !value;

        }

    },

    staticMoving: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .staticMoving has been deprecated. Use .enableDamping instead.");
            return !this.enableDamping;

        },

        set: function(value) {

            console.warn("VG.CameraControllerOrbit: .staticMoving has been deprecated. Use .enableDamping instead.");
            this.enableDamping = !value;

        }

    },

    dynamicDampingFactor: {

        get: function() {

            console.warn("VG.CameraControllerOrbit: .dynamicDampingFactor has been renamed. Use .dampingFactor instead.");
            return this.dampingFactor;

        },

        set: function(value) {

            console.warn("VG.CameraControllerOrbit: .dynamicDampingFactor has been renamed. Use .dampingFactor instead.");
            this.dampingFactor = value;

        }

    }
};

export {CameraControllerOrbit};
