/**
 * @author mrdoob / http://mrdoob.com/
 * @author alteredq / http://alteredqualia.com/
 * @author paulirish / http://paulirish.com/
 */

import {EventDispatcher} from "../base/EventDispatcher.js";
import { MOBILE_CLIENT } from "../settings.js";

function FirstPersonControls( object, domCanvasElement, domElement) {

	this.object = object;
	this.target = new THREE.Vector3( 0, 0, 0 );
	this.object.lookAt(this.target);

	this.domCanvasElement = ( domCanvasElement !== undefined ) ? domCanvasElement : document;

	this.enabled = true;
	this.moveEnabled = true;

	this.movementSpeed = 1.0;
	this.lookSpeed = 0.005;

	this.lookVertical = true;
	this.autoForward = false;

	this.activeLook = true;

	this.heightSpeed = false;
	this.heightCoef = 1.0;
	this.heightMin = 0.0;
	this.heightMax = 1.0;

	this.constrainVertical = false;
	this.verticalMin = 0;
	this.verticalMax = Math.PI;

	this.autoSpeedFactor = 0.0;

	this.mouseX = 0;
	this.mouseY = 0;

	this.lat = 0;
	this.lon = 0;
	this.phi = 0;
	this.theta = 0;

	this.mouseMoveDirection = 0

	this.moveForward = false;
	this.moveBackward = false;
	this.moveLeft = false;
	this.moveRight = false;

	this.previousTouch = null;

	this.mouseDragOn = false;

	this.mouseIsMove = false;

	this.viewHalfX = 0;
	this.viewHalfY = 0;

	/* для обеспечения движения */
	this.direction = new THREE.Vector3();
	this._vector = new THREE.Vector3();

	this.lastEvent = null;

	this.keys = {LEFT: "KeyA", UP: "KeyE", RIGHT: "KeyD", DOWN: "KeyQ", FORWARD: "KeyW", BACKWARD: "KeyS",
		LEFT_ARROW: "ArrowLeft", RIGHT_ARROW: "ArrowRight", UP_ARROW: "ArrowUp", DOWN_ARROW: "ArrowDown"};

	const _euler = new THREE.Euler( 0, 0, 0, 'YXZ' );

	if ( this.domCanvasElement !== document ) {

		this.domCanvasElement.setAttribute( 'tabindex', - 1 );

	}

	this.allowMove = function( value ) {

		this.moveEnabled = value;
	}

	//

	this.handleResize = function () {

		if ( this.domCanvasElement === document ) {

			this.viewHalfX = window.innerWidth / 2;
			this.viewHalfY = window.innerHeight / 2;

		} else {

			this.viewHalfX = this.domCanvasElement.offsetWidth / 2;
			this.viewHalfY = this.domCanvasElement.offsetHeight / 2;

		}

	};

	this.onMouseDown = function ( {event} ) {
		
		if ( this.domCanvasElement !== document ) {

			this.domCanvasElement.focus();

		}

		event.preventDefault();
		event.stopPropagation();
		

		if ( event.button == 2 ) {
			this.mouseDragOn = true;
			this.mouseMoveDirection = 0;
		}	
	};

	this.onMouseUp = function ( {event} ) {

		this.previousTouch = null;
		event.preventDefault();
		event.stopPropagation();

		if ( event.button == 2 ) {
			this.mouseDragOn = false;
			this.mouseIsMove = false;
			if (this.mouseMoveDirection) {
				this.mouseMoveDirection = 0;
				this.moveBackward = false;
				this.moveForward = false;
			}
		}
	};

	this.onMouseMove = function ( {event} ) {

		if (MOBILE_CLIENT) {
			let touch;
			if (event.touches[0].target.id === "joystick") {
				touch = event.touches[1];
			} else {
				touch = event.touches[0];
			}
			
			if (this.previousTouch && touch) {
				event.movementX = touch.pageX - this.previousTouch.pageX;
				event.movementY = touch.pageY - this.previousTouch.pageY;

				const movementX = event.movementX || event.mozMovementX || event.webkitMovementX || 0;
				const movementY = event.movementY || event.mozMovementY || event.webkitMovementY || 0;

				_euler.setFromQuaternion( this.object.quaternion );

				_euler.y -= movementX * 0.008;
				_euler.x -= movementY * 0.008;

				// _euler.x = Math.max( Math.PI / 2 - Math.PI, Math.min( Math.PI / 2 - 0, _euler.x ) );
				if (_euler.x > -1.5 && _euler.x < 1.5) {
					this.object.quaternion.setFromEuler( _euler );
					// EventDispatcher.send("CameraController.update", this.object);
				}
			};

			this.previousTouch = touch;
			
		}
	
		if ( document.pointerLockElement === domElement) {

			let movementX = event.movementX || event.mozMovementX || event.webkitMovementX || 0;
			let movementY = event.movementY || event.mozMovementY || event.webkitMovementY || 0;


			if (this.lastEvent && event.timeStamp - this.lastEvent.timeStamp > 2000) {
				movementX = 0
				movementY = 0
			}


			_euler.setFromQuaternion( this.object.quaternion );

			_euler.y -= movementX / 500;
			
			if (!this.mouseDragOn)
				_euler.x -= movementY / 500;

			else {
				this.mouseMoveDirection -= movementY * 1.5;
				let sg = Math.sign(this.mouseMoveDirection) 

				this.mouseMoveDirection = sg * Math.min(Math.abs(this.mouseMoveDirection), 250)
				this.moveForward = this.mouseMoveDirection > 0 ? true : false;
				this.moveBackward = this.mouseMoveDirection < 0 ? true : false;
				this.isMoving()
			}

			// _euler.x = Math.max( Math.PI / 2 - Math.PI, Math.min( Math.PI / 2 - 0, _euler.x ) );
			if (_euler.x > -1.5 && _euler.x < 1.5) {
				this.object.quaternion.setFromEuler( _euler );
				// EventDispatcher.send("CameraController.update", this.object);
				EventDispatcher.send("ViewHelper.update", this.object);
			}
		}
		this.lastEvent = event
	};


	this.onKeyDown = function ( event ) {

		//event.preventDefault();

		switch ( event.keyCode ) {

			case 38: /*up*/
			case 87: /*W*/ this.moveForward = true; break;

			case 37: /*left*/
			case 65: /*A*/ this.moveLeft = true; break;

			case 40: /*down*/
			case 83: /*S*/ this.moveBackward = true; break;

			case 39: /*right*/
			case 68: /*D*/ this.moveRight = true; break;

			case 82: /*R*/ this.moveUp = true; break;
			case 70: /*F*/ this.moveDown = true; break;

		}

	};

	this.onKeyUp = function ( event ) {

		switch ( event.keyCode ) {

			case 38: /*up*/
			case 87: /*W*/ this.moveForward = false; break;

			case 37: /*left*/
			case 65: /*A*/ this.moveLeft = false; break;

			case 40: /*down*/
			case 83: /*S*/ this.moveBackward = false; break;

			case 39: /*right*/
			case 68: /*D*/ this.moveRight = false; break;

			case 82: /*R*/ this.moveUp = false; break;
			case 70: /*F*/ this.moveDown = false; break;

		}

	};

	this.onKeyDownMoveForward = function () {
		this.moveForward = true;
		this.isMoving();
	}

	this.onKeyUpMoveForward = function () {
		this.moveForward = false;
		this.isMoving();
	}

	this.onKeyDownMoveLeft = function () {
		this.moveLeft = true;
		this.isMoving();
	}

	this.onKeyUpMoveLeft = function () {
		this.moveLeft = false;
		this.isMoving();
	}

	this.onKeyDownMoveBackward = function () {
		this.moveBackward = true;
		this.isMoving();
	}

	this.onKeyUpMoveBackward = function () {
		this.moveBackward = false;
		this.isMoving();
	}

	this.onKeyDownMoveRight = function () {
		this.moveRight = true;
		this.isMoving();
	}

	this.onKeyUpMoveRight = function () {
		this.moveRight = false;
		this.isMoving();
	}

	this.onKeyDownMoveUp = function (e) {
		this.moveUp = true;
		this.isMoving();
	}

	this.onKeyUpMoveUp = function (e) {
		this.moveUp = false;
		this.isMoving();
	}

	this.onKeyDownMoveDown = function (e) {
		this.moveDown = true;
		this.isMoving();
	}

	this.onKeyUpMoveDown = function (e) {
		this.moveDown = false;
		this.isMoving();
	}

	this.onKeyDownRotateUp = function (e) {
		this.rotateUp = true;
		_euler.setFromQuaternion( this.object.quaternion );
		this.isMoving();
	}

	this.onKeyUpRotateUp = function (e) {
		this.rotateUp = false;
		this.isMoving();
	}

	this.onKeyDownRotateDown = function (e) {
		this.rotateDown = true;
		_euler.setFromQuaternion( this.object.quaternion );
		this.isMoving();
	}

	this.onKeyUpRotateDown = function (e) {
		this.rotateDown = false;
		this.isMoving();
	}

	this.onKeyDownRotateLeft = function (e) {
		this.rotateLeft = true;
		_euler.setFromQuaternion( this.object.quaternion );
		this.isMoving();
	}

	this.onKeyUpRotateLeft = function (e) {
		this.rotateLeft = false;
		this.isMoving();
	}

	this.onKeyDownRotateRight = function (e) {
		this.rotateRight = true;
		_euler.setFromQuaternion( this.object.quaternion );
		this.isMoving();
	}

	this.onKeyUpRotateRight = function (e) {
		this.rotateRight = false;
		this.isMoving();
	}

	this.isMoving = () => {
        if (
            this.moveForward ||
            this.moveLeft ||
            this.moveBackward ||
            this.moveRight ||
            this.moveUp ||
            this.moveDown ||
            this.rotateLeft ||
            this.rotateRight  ||
			this.rotateUp ||
			this.rotateDown
            ) {
            this.isMovement = true;
        } else {
            this.isMovement = false;
        }
    }

	this.update = delta => {
		if (this.isMovement) {
            this._update(delta);
        }
	}
	this._update = function(delta) {
		let startPosition = this.object.position.clone()

		if ( this.enabled === false ) return;

		if ( this.heightSpeed ) {

			var y = THREE.Math.clamp( this.object.position.y, this.heightMin, this.heightMax );
			var heightDelta = y - this.heightMin;

			this.autoSpeedFactor = delta * ( heightDelta * this.heightCoef );

		} else {

			this.autoSpeedFactor = 0.0;

		}

		var actualMoveSpeed = delta * this.movementSpeed;
		const factor = 1/delta;

		if (this.mouseDragOn)
			actualMoveSpeed = delta * Math.abs(this.mouseMoveDirection) / 8;


		if (this.rotateUp || this.rotateDown || this.rotateLeft || this.rotateRight){
			_euler.x += (this.rotateUp ? delta : this.rotateDown ? -delta : 0)
			if (_euler.x < -1.5)
					_euler.x = -1.5;
			if (_euler.x > 1.5)
					_euler.x = 1.5;;

			_euler.y += (this.rotateLeft ? delta : this.rotateRight ? -delta : 0)
			this.object.quaternion.setFromEuler( _euler );
			EventDispatcher.send("ViewHelper.update", this.object);
		}
		

		if ((this.moveForward && (this.moveLeft || this.moveRight)) || (this.moveBackward && (this.moveLeft || this.moveRight))) {
			actualMoveSpeed *= 0.7;
		}

		if (MOBILE_CLIENT) {
			const joyStickDelta = Math.max(EventDispatcher.query("JoyStickEventsHandler.getState").x, EventDispatcher.query("JoyStickEventsHandler.getState").y);
			if (joyStickDelta > 0) {
				actualMoveSpeed *= joyStickDelta;
			}
		}

		if ( this.moveForward || ( this.autoForward && ! this.moveBackward ) ) {

			const velocity = new THREE.Vector3();

			velocity.z -= velocity.z * actualMoveSpeed;


			this.direction.z = Number( this.moveForward  ) - Number( this.moveBackward );
			this.direction.normalize();
			velocity.z -= this.direction.z * actualMoveSpeed *factor;
			this._vector.setFromMatrixColumn( this.object.matrix, 0 );
			this._vector.crossVectors( this.object.up, this._vector );

			this.object.position.addScaledVector( this._vector, - velocity.z * delta  );
		}
		if ( this.moveBackward ) {

			const velocity = new THREE.Vector3()
			velocity.z += velocity.z * velocity.z * actualMoveSpeed;
			this.direction.z = Number( this.moveBackward ) - Number( this.moveForward );
			this.direction.normalize();
			velocity.z += this.direction.z * actualMoveSpeed*factor;
			this._vector.setFromMatrixColumn( this.object.matrix, 0 );
			this._vector.crossVectors( this.object.up, this._vector );
			this.object.position.addScaledVector( this._vector, -velocity.z * delta  );
		} 

		if ( this.moveLeft ) {
			this.object.translateX( - actualMoveSpeed );
		} 
		if ( this.moveRight ) {
			this.object.translateX( actualMoveSpeed );
		}

		if ( this.moveUp ) {
			this.object.position.y += actualMoveSpeed;
		} 
		if ( this.moveDown ) {
			this.object.position.y -= actualMoveSpeed;
		} 

		if (this.moveForward || this.moveBackward || this.moveLeft || this.moveRight || this.moveUp || this.moveDown || this.rotateLeft || this.rotateRight) {
			let direction = this.object.position.clone().sub(startPosition).normalize()
			EventDispatcher.send("Camera.update", this.object, this.object.position, {x: 0, y: 0}, direction);
		}

		if (!this.moveEnabled)
			this.object.position.copy(startPosition);

		// if ( this.moveForward || ( this.autoForward && ! this.moveBackward ) ) this.object.translateZ( - ( actualMoveSpeed + this.autoSpeedFactor ) );
		// if ( this.moveBackward ) this.object.translateZ( actualMoveSpeed );

		// if ( this.moveLeft ) this.object.translateX( - actualMoveSpeed );
		// if ( this.moveRight ) this.object.translateX( actualMoveSpeed );

		// if ( this.moveUp ) this.object.translateY( actualMoveSpeed );
		// if ( this.moveDown ) this.object.translateY( - actualMoveSpeed );

		if (this.object.position.y < 1) {
			this.object.position.y = 1;
		}

		var verticalLookRatio = 1;

		if ( this.constrainVertical ) {

			verticalLookRatio = Math.PI / ( this.verticalMax - this.verticalMin );

		}

	};

	function contextmenu( event ) {

		event.preventDefault();

	}

	this.dispose = function() {

		this.domCanvasElement.removeEventListener( 'contextmenu', contextmenu, false );
		this.domCanvasElement.removeEventListener( 'mousedown', _onMouseDown, false );
		this.domCanvasElement.removeEventListener( 'mousemove', _onMouseMove, false );
		this.domCanvasElement.removeEventListener( 'mouseup', _onMouseUp, false );

		window.removeEventListener( 'keydown', _onKeyDown, false );
		window.removeEventListener( 'keyup', _onKeyUp, false );

	};

	this.removeEvent = function() {
		EventDispatcher.unbind("mouse.move", _onMouseMove);
		EventDispatcher.unbind("mouse.down", _onMouseDown);
		EventDispatcher.unbind("mouse.up", _onMouseUp);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.FORWARD, this.onKeyDownMoveForward);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.FORWARD, this.onKeyUpMoveForward);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.BACKWARD, this.onKeyDownMoveBackward);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.BACKWARD, this.onKeyUpMoveBackward);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.LEFT, this.onKeyDownMoveLeft);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.LEFT, this.onKeyUpMoveLeft);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.RIGHT, this.onKeyDownMoveRight);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.RIGHT, this.onKeyUpMoveRight);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.UP, this.onKeyDownMoveUp);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.UP, this.onKeyUpMoveUp);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.DOWN, this.onKeyDownMoveDown);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.DOWN, this.onKeyUpMoveDown);

		EventDispatcher.unbind("keyboard.keydown." + this.keys.UP_ARROW, this.onKeyDownMoveForward);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.UP_ARROW, this.onKeyUpMoveForward);
		EventDispatcher.unbind("keyboard.keydown." + this.keys.DOWN_ARROW, this.onKeyDownMoveBackward);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.DOWN_ARROW, this.onKeyUpMoveBackward);

		EventDispatcher.unbind("keyboard.keydown." + this.keys.LEFT_ARROW, this.onKeyDownRotateLeft); 
		EventDispatcher.unbind("keyboard.keyup." + this.keys.LEFT_ARROW, this.onKeyUpRotateLeft); 

		EventDispatcher.unbind("keyboard.keydown." + this.keys.RIGHT_ARROW, this.onKeyDownRotateRight);
		EventDispatcher.unbind("keyboard.keyup." + this.keys.RIGHT_ARROW, this.onKeyUpRotateRight);


	};

	var _onMouseMove = bind( this, this.onMouseMove );
	var _onMouseDown = bind( this, this.onMouseDown );
	var _onMouseUp = bind( this, this.onMouseUp );
	var _onKeyDown = bind( this, this.onKeyDown );
	var _onKeyUp = bind( this, this.onKeyUp );

	this.initEvent = function() {

		EventDispatcher.bind("mouse.move", this, _onMouseMove);
		EventDispatcher.bind("mouse.down", this, _onMouseDown);
		EventDispatcher.bind("mouse.up", this, _onMouseUp);

		EventDispatcher.bind("Camera.allowMove", this, this.allowMove);

		EventDispatcher.bind("keyboard.keydown." + this.keys.FORWARD, this, this.onKeyDownMoveForward);
		EventDispatcher.bind("keyboard.keyup." + this.keys.FORWARD, this, this.onKeyUpMoveForward);
		EventDispatcher.bind("keyboard.keydown." + this.keys.BACKWARD, this, this.onKeyDownMoveBackward);
		EventDispatcher.bind("keyboard.keyup." + this.keys.BACKWARD, this, this.onKeyUpMoveBackward);
		EventDispatcher.bind("keyboard.keydown." + this.keys.LEFT, this, this.onKeyDownMoveLeft);
		EventDispatcher.bind("keyboard.keyup." + this.keys.LEFT, this, this.onKeyUpMoveLeft);
		EventDispatcher.bind("keyboard.keydown." + this.keys.RIGHT, this, this.onKeyDownMoveRight);
		EventDispatcher.bind("keyboard.keyup." + this.keys.RIGHT, this, this.onKeyUpMoveRight);
		EventDispatcher.bind("keyboard.keydown." + this.keys.UP, this, this.onKeyDownMoveUp);
		EventDispatcher.bind("keyboard.keyup." + this.keys.UP, this, this.onKeyUpMoveUp);
		EventDispatcher.bind("keyboard.keydown." + this.keys.DOWN, this, this.onKeyDownMoveDown);
		EventDispatcher.bind("keyboard.keyup." + this.keys.DOWN, this, this.onKeyUpMoveDown);

		EventDispatcher.bind("keyboard.keydown." + this.keys.UP_ARROW, this, this.onKeyDownRotateUp);
		EventDispatcher.bind("keyboard.keyup." + this.keys.UP_ARROW, this, this.onKeyUpRotateUp);

		EventDispatcher.bind("keyboard.keydown." + this.keys.DOWN_ARROW, this, this.onKeyDownRotateDown);
		EventDispatcher.bind("keyboard.keyup." + this.keys.DOWN_ARROW, this, this.onKeyUpRotateDown);

		EventDispatcher.bind("keyboard.keydown." + this.keys.LEFT_ARROW, this, this.onKeyDownRotateLeft); 
		EventDispatcher.bind("keyboard.keyup." + this.keys.LEFT_ARROW, this, this.onKeyUpRotateLeft); 

		EventDispatcher.bind("keyboard.keydown." + this.keys.RIGHT_ARROW, this, this.onKeyDownRotateRight);
		EventDispatcher.bind("keyboard.keyup." + this.keys.RIGHT_ARROW, this, this.onKeyUpRotateRight);
	}

	function bind( scope, fn ) {

		return function () {

			fn.apply( scope, arguments );

		};

	}

	this.handleResize();
};

export {FirstPersonControls};