import { EventDispatcher } from "../base/EventDispatcher";

class ViewHelper extends THREE.Object3D {

	constructor(editorCamera) {

		super();
		//rgba(37, 47, 56, 1)
		//rgb(23, 29, 34)
		//rgb(9, 11, 13)
		const color1 = new THREE.Color('rgb(37, 47, 56)');

		const geometry = new THREE.BoxGeometry( 0.8, 0.04, 0.04 ).translate( 0.4, 0, 0 );

		const xAxis = new THREE.Mesh( geometry, getAxisMaterial( color1 ) );
        xAxis.ignoreRaycaster = true;
		const yAxis = new THREE.Mesh( geometry, getAxisMaterial( color1 ) );
        yAxis.ignoreRaycaster = true;
		const zAxis = new THREE.Mesh( geometry, getAxisMaterial( color1 ) );
        zAxis.ignoreRaycaster = true;

		yAxis.rotation.z = Math.PI / 2;
		zAxis.rotation.y = - Math.PI / 2;

		this.add( xAxis );
		this.add( zAxis );
		this.add( yAxis );

		const posXAxisHelper = new THREE.Sprite( getSpriteMaterial( color1, 'X' ) );
		posXAxisHelper.userData.type = 'posX';
		const posYAxisHelper = new THREE.Sprite( getSpriteMaterial( color1, 'Y' ) );
		posYAxisHelper.userData.type = 'posY';
		const posZAxisHelper = new THREE.Sprite( getSpriteMaterial( color1, 'Z' ) );
		posZAxisHelper.userData.type = 'posZ';
		// const negXAxisHelper = new THREE.Sprite( getSpriteMaterial( color1 ) );
		// negXAxisHelper.userData.type = 'negX';
		// const negYAxisHelper = new THREE.Sprite( getSpriteMaterial( color2 ) );
		// negYAxisHelper.userData.type = 'negY';
		// const negZAxisHelper = new THREE.Sprite( getSpriteMaterial( color3 ) );
		// negZAxisHelper.userData.type = 'negZ';

		posXAxisHelper.position.x = 1;
		posYAxisHelper.position.y = 1;
		posZAxisHelper.position.z = 1;
		// negXAxisHelper.position.x = - 1;
		// negXAxisHelper.scale.setScalar( 0.8 );
		// negYAxisHelper.position.y = - 1;
		// negYAxisHelper.scale.setScalar( 0.8 );
		// negZAxisHelper.position.z = - 1;
		// negZAxisHelper.scale.setScalar( 0.8 );

        posXAxisHelper.ignoreRaycaster = true;
		posYAxisHelper.ignoreRaycaster = true;
		posZAxisHelper.ignoreRaycaster = true;
		
		// negXAxisHelper.ignoreRaycaster = true;
		// negYAxisHelper.ignoreRaycaster = true;
		// negZAxisHelper.ignoreRaycaster = true;

		this.add( posXAxisHelper );
		this.add( posYAxisHelper );
		this.add( posZAxisHelper );
		// this.add( negXAxisHelper );
		// this.add( negYAxisHelper );
		// this.add( negZAxisHelper );

        this.scale.set(0.1, 0.1, 0.1);

		editorCamera.add(this);

		function addAxisHelper() {
			editorCamera.add(this);
		}

		function removeAxisHelper() {
			editorCamera.remove(this);
		}

		this.position.set((editorCamera.aspect * 0.83 - 0.2), -0.66, -2);
        
        this.updatePos = (o) => {
			this.quaternion.copy(o.quaternion).invert();
			this.position.set((editorCamera.aspect * 0.83 - 0.2), -0.66, -2);
        }

        EventDispatcher.bind("Camera.update", this, this.updatePos);
		EventDispatcher.bind("ViewHelper.update", this, this.updatePos);
		EventDispatcher.bind("ViewHelper.add", this, addAxisHelper);
		EventDispatcher.bind("ViewHelper.remove", this, removeAxisHelper);

		function getAxisMaterial( color ) {

			const m = new THREE.MeshBasicMaterial( { color: color, toneMapped: false } );
			m.depthTest = false;
			m.depthWrite = false;
			m.transparent = true;
			m.needsUpdate = true;
			return m;

		}

		function getSpriteMaterial( color, text = null ) {

			const canvas = document.createElement( 'canvas' );
			canvas.width = 64;
			canvas.height = 64;

			const context = canvas.getContext( '2d' );
			context.beginPath();
			context.arc( 32, 32, 16, 0, 2 * Math.PI );
			context.closePath();
			context.fillStyle = color.getStyle();
			// context.fillStyle = "rgb(37, 47, 56)";
			context.fill();

			if ( text !== null ) {

				context.font = '24px Arial';
				context.textAlign = 'center';
				context.fillStyle = '#FFFFFF';
				context.fillText( text, 32, 41 );

			}

			const texture = new THREE.CanvasTexture( canvas );
			const m = new THREE.SpriteMaterial( { map: texture, toneMapped: false } );
			m.depthTest = false;
			m.depthWrite = false;
			m.transparent = true;
			m.needsUpdate = true;
			return m;

		}

		window.addEventListener("resize", () => {
			this.updatePos(editorCamera);
		});

	}

}

export { ViewHelper };