import {EventDispatcher} from "../base/EventDispatcher.js";

import {Container} from "../ui/Container";

function JoyStickUI() {
    Container.apply(this, arguments);

    EventDispatcher.bind("JoyStick.show", this, this.show);
    EventDispatcher.bind("JoyStick.hide", this, this.hide);

    const tempModalMenu = document.getElementById('temp__joystick');
    this.modalWrapper = tempModalMenu.content.getElementById('joystick-wrapper').cloneNode(true);
    this.modalWrapper.classList.add("mobile");
    this.add(this.modalWrapper);
    arguments[1].appendChild(this.view);

    // this.hide();
    this.modalOpen = false;
}

JoyStickUI.prototype = Object.create(Container.prototype);
JoyStickUI.constructor = JoyStickUI;


const JoyStickEventsHandler = function (scene, domElement) {
    this.scene = scene;
    this.domElement = domElement;

    this.createUI();
    this.initJoyStick();

    this.joyStickState = {
        x: 0,
        y: 0,
    }

    EventDispatcher.bind("JoyStickEventsHandler.getState", this, function() {
        return this.joyStickState;
    });
}

JoyStickEventsHandler.prototype = {

    constructor: JoyStickEventsHandler,

    createUI() {
        this.UI = new JoyStickUI("joystickUI", this.domElement);
    },

    initJoyStick() {
        this.joystick = new JoyStick("joystick-wrapper", {
            internalFillColor: "rgb(74 114 219 / 10%)",
            internalStrokeColor: "#4A72DB",
            externalLineWidth: 1,
            externalStrokeColor: "#4A72DB",
        }, e => {
            this.joyStickState.x = Math.abs(+e.x) / 100;
            this.joyStickState.y = Math.abs(+e.y) / 100;
            switch (e.cardinalDirection) {
                case "N":
                    EventDispatcher.send("keyboard.keyup.KeyS");
                    EventDispatcher.send("keyboard.keyup.KeyA");
                    EventDispatcher.send("keyboard.keyup.KeyD");
                    EventDispatcher.send("keyboard.keydown.KeyW");
                    break;
                case "NW":
                    EventDispatcher.send("keyboard.keyup.KeyS");
                    EventDispatcher.send("keyboard.keyup.KeyD");
                    EventDispatcher.send("keyboard.keydown.KeyW");
                    EventDispatcher.send("keyboard.keydown.KeyA");
                    break;
                case "NE":
                    EventDispatcher.send("keyboard.keyup.KeyA");
                    EventDispatcher.send("keyboard.keyup.KeyS");
                    EventDispatcher.send("keyboard.keydown.KeyW");
                    EventDispatcher.send("keyboard.keydown.KeyD");
                    break;
                case "S":
                    EventDispatcher.send("keyboard.keyup.KeyW");
                    EventDispatcher.send("keyboard.keyup.KeyA");
                    EventDispatcher.send("keyboard.keyup.KeyD");
                    EventDispatcher.send("keyboard.keydown.KeyS");
                    break;
                case "SW":
                    EventDispatcher.send("keyboard.keyup.KeyW");
                    EventDispatcher.send("keyboard.keyup.KeyD");
                    EventDispatcher.send("keyboard.keydown.KeyS");
                    EventDispatcher.send("keyboard.keydown.KeyA");
                    break;
                case "SE":
                    EventDispatcher.send("keyboard.keyup.KeyW");
                    EventDispatcher.send("keyboard.keyup.KeyA");
                    EventDispatcher.send("keyboard.keydown.KeyS");
                    EventDispatcher.send("keyboard.keydown.KeyD");
                    break;
                case "W":
                    EventDispatcher.send("keyboard.keyup.KeyW");
                    EventDispatcher.send("keyboard.keyup.KeyD");
                    EventDispatcher.send("keyboard.keyup.KeyS");
                    EventDispatcher.send("keyboard.keydown.KeyA");
                    break;
                case "E":
                    EventDispatcher.send("keyboard.keyup.KeyA");
                    EventDispatcher.send("keyboard.keyup.KeyW");
                    EventDispatcher.send("keyboard.keyup.KeyS");
                    EventDispatcher.send("keyboard.keydown.KeyD");
                    break;
                case "C":
                    EventDispatcher.send("keyboard.keyup.KeyW");
                    EventDispatcher.send("keyboard.keyup.KeyS");
                    EventDispatcher.send("keyboard.keyup.KeyA");
                    EventDispatcher.send("keyboard.keyup.KeyD");
                    break;

                default:
                    break;
            }
        });
    },

}

export {JoyStickEventsHandler};