import {EventDispatcher} from "../base/EventDispatcher";
import {CSM} from "./csm/CSM";
import {DEBUG} from "../settings";
import {CSMHelper} from "./csm/CSMHelper";

function ShadowController(props) {
    this.csmHelper = null;

    this.camera = EventDispatcher.query('Engine.get.camera');
    const scene = EventDispatcher.query("Engine.get.scene");
    const _this = this;

    this.enabled = true;

    this.controller = new CSM({
        maxFar: 200,
        lightFar: 1000,
        cascades: props.cascades,
        shadowMapSize: props.mapSize,
        lightDirection: new THREE.Vector3(-0.2, -1, -0.2),
        camera: this.camera,
        parent: scene,
        mode: 'practical', //uniform (linear), logarithmic, practical or custom
        fade: true,
        lightMargin: 100,
        shadowBias: [0.00007, 0.000001, 0.000001, 0.000001]
    });

    if (DEBUG) {
        this.csmHelper = new CSMHelper(this.controller);
        this.csmHelper.visible = true;
        EventDispatcher.send("Scene.add", this.csmHelper);
    }

    _this.controller.lights.forEach(el => {
        el.shadow.autoUpdate = false
    });

    this.counters = {
        0: {
            current: 0,
            required: 5 //8
        },
        1: {
            current: 0,
            required: 21
        },
        2: {
            current: 0,
            required: 40
        },
        3: {
            current: 0,
            required: 60
        },
    };

    EventDispatcher.bind("ShadowController.enabled", this, function (value) {
        _this.enabled = value;

        for (let i = 0; i < _this.controller.lights.length; i++) {
            _this.controller.lights[i].castShadow = value;
        }
    })

    EventDispatcher.bind("ShadowController.apply", this, this.apply);

    EventDispatcher.bind("ShadowController.setDistance", this, function (dist) {
        _this.controller.maxFar = dist + 100;
    })

    EventDispatcher.bind("ShadowController.setSunPosition", this, this.setSunPosition);

    EventDispatcher.send("Engine.add", this);
}

ShadowController.constructor = ShadowController;

ShadowController.prototype.apply = function (mesh) {
    this.controller.setupMaterial(mesh.material);
};

ShadowController.prototype.update = function (dt) {
    if (!this.enabled) return;

    for (let i = 0; i < this.controller.lights.length; i++) {

        if (this.counters[i].current === this.counters[i].required) {
            this.controller.lights[i].shadow.needsUpdate = true;
            this.counters[i].current = 0

            if (DEBUG) {
                this.csmHelper.update();
                this.csmHelper.updateVisibility();
            }
            this.controller.update(this.camera.matrix);

        } else {
            this.counters[i].current++;
        }
    }

}

ShadowController.prototype.setSunPosition = function (pos) {
    this.controller.lightDirection = pos.normalize();
}

export {ShadowController}
