import {EventDispatcher} from "../../base/EventDispatcher";
import {SHADOW_QUALITY} from "../../settings";

function BaseShadow(light) {
    this.light = light;
    this.light.castShadow = true;

    this.light.shadow.camera.near = 0.5; // default
    this.light.shadow.camera.far = 1000;


    this.light.shadow.radius = 3;
    this.light.shadow.blurSamples = 16;
    this.light.shadow.normalBias = 1;

    this.shadowCameraOffset = this.light.position.clone();

    //  this.light.shadow.bias = -0.0002;
    // this.updateShadowMap(2);

    /*this.cameraHelper = new THREE.CameraHelper(this.light.shadow.camera);
    EventDispatcher.send("Scene.add", this.cameraHelper);*/

    // const cameraHelper = new THREE.CameraHelper(this.light.shadow.camera);
    // EventDispatcher.send("Scene.add", cameraHelper);

    this.activeLevel;
    EventDispatcher.send("Scene.add", this.light.target);
    EventDispatcher.bind("Camera.update", this, this.changeLevel);
}

BaseShadow.prototype = Object.create(BaseShadow.prototype);
BaseShadow.constructor = BaseShadow;

BaseShadow.prototype.changeLevel = function (camera, target) {
    this.light.target.position.set(camera.position.x, 0, camera.position.z);
    this.light.target.updateMatrixWorld();
    this.light.position.set(camera.position.x, 0, camera.position.z).add(this.shadowCameraOffset);

    function roundTo(n, x) {
        return Math.round(x / n);
    }

    const dist = (target) ? camera.position.distanceTo(target) : camera.position.y;

    let level = roundTo(Math.abs(dist), 45);
    level = (level > 2) ? 2 : level;

    if (this.activeLevel !== level) {

        this.updateShadowMap(level);
    }

};

BaseShadow.prototype.updateShadowMap = function (level) {
    if (!this.light.shadow.map) return;

    this.light.shadow.map.dispose();
    this.light.shadow.map = null;

    this.light.shadow.mapSize.width = SHADOW_QUALITY[level].mapSize;
    this.light.shadow.mapSize.height = SHADOW_QUALITY[level].mapSize;

    this.light.shadow.camera.bottom = -SHADOW_QUALITY[level].bbox;
    this.light.shadow.camera.top = SHADOW_QUALITY[level].bbox;
    this.light.shadow.camera.left = -SHADOW_QUALITY[level].bbox;
    this.light.shadow.camera.right = SHADOW_QUALITY[level].bbox;

    this.activeLevel = level;
};

export {BaseShadow}