import {EventDispatcher} from "../base/EventDispatcher";
import {TransformControls} from "./controllers/TransformControls.js";


const TransformTool = function(scene, camera, domCanvasElement, domElement) {
    this.scene = scene;
    this.camera = camera;
    this.domElement = domElement;
    this.domCanvasElement = domCanvasElement;
    this.state = null;

    this.mode = {
        TRANSLATE: "TRANSLATE",
        ROTATE: "ROTATE",
        SCALE: "SCALE",
    }

    this.transformController = new TransformControls(this.camera, domCanvasElement);
    EventDispatcher.send("Scene.add", this.transformController);

    this.transformController.addEventListener('dragging-changed', function (e) {
        if (e.value) {
            EventDispatcher.send("Input.disable");
        } else {
            EventDispatcher.send("Input.enable");
        }
    });

    EventDispatcher.bind("TransformTool.attach", this, function(mesh) {
        this.transformController.attach(mesh);
    });

    EventDispatcher.bind("TransformTool.setMode", this, this.setMode);

}

TransformTool.prototype = {

    constructor: TransformTool,

    setMode: function(mode) {
        if (mode === this.mode.TRANSLATE) {
            this.transformController.setMode(this.mode.TRANSLATE.toLowerCase());
        } else if (mode === this.mode.ROTATE) {
            this.transformController.setMode(this.mode.ROTATE.toLowerCase());
        } else if (mode === this.mode.SCALE) {
            this.transformController.setMode(this.mode.SCALE.toLowerCase());
        } else {
            
        }
    },

}

export {TransformTool};